==> raml-client-2.0.4.tgz: package/src/generator/documentation.js <==
module.exports = function buildDocumentation(name, ast) {
  const documentation = [];

  documentation.push('/* RAML Client API calls */');
  documentation.push(`declare var ${name}: {`);
  documentation.push('  /**');
  documentation.push('   * Make an API request to a custom URL.');
  documentation.push('   *');
  documentation.push('   * @param {string} url - Provide a url relative to the base URI.');
  documentation.push('   * @param {any} data - Provide a data object to replace template tags in the url.');
  documentation.push('   */');
  documentation.push('  (url: string, data?): {');
  documentation.push('    delete(body?: any, options?: any, async?: boolean): any,');
  documentation.push('    head(query?: any, options?: any, async?: boolean): any,');
  documentation.push('    get(query?: any, options?: any, async?: boolean): any,');
  documentation.push('    patch(body?: any, options?: any, async?: boolean): any,');
  documentation.push('    post(body?: any, options?: any, async?: boolean): any,');
  documentation.push('    put(body?: any, options?: any, async?: boolean): any');
  documentation.push('  };');

  buildMethods(ast.methods, documentation, 1);
  buildResources(ast.resources, documentation, 1);

  documentation.push('};');

  return documentation.join('\n');
};

function buildResources(resources, documentation, identation) {
  if (resources) {
    Object.keys(resources).forEach((resource) => {
      if (resource.startsWith('{')) {
        let methodName = resource.substring(1, resource.length - 1);

        if (methodName[0] === '+') {
          methodName = methodName.substring(1);
        }

        documentation.push(`${buildIdentation(identation)}${methodName}(${methodName}: string): {`);

        buildMethods(resources[resource].methods, documentation, identation + 1);
        buildResources(resources[resource].resources, documentation, identation + 1);

        documentation.push(`${buildIdentation(identation)}};`);
      } else {
        documentation.push(`${buildIdentation(identation)}${resource}: {`);

        buildMethods(resources[resource].methods, documentation, identation + 1);
        buildResources(resources[resource].resources, documentation, identation + 1);

        documentation.push(`${buildIdentation(identation)}};`);
      }
    });

    return documentation;
  }

  return documentation;
}

function buildMethods(methods, documentation, identation) {
  if (methods) {
    Object.keys(methods).forEach((method) => {
      if (methods[method].queryParameters) {
        const params = Object.keys(methods[method].queryParameters);
        documentation.push(`${buildIdentation(identation)}/**`);
        documentation.push(`${buildIdentation(identation)} * ${methods[method].description}`);
        documentation.push(`${buildIdentation(identation)} *`);
        params.forEach((param) => {
          const info = methods[method].queryParameters[param];

          documentation.push(`${buildIdentation(identation)} * @param {${getType(info)}} ${info.name}${info.required ? '?' : ''} - ${info.description}`);
        });
        documentation.push(`${buildIdentation(identation)}*/`);
        documentation.push(`${buildIdentation(identation)}${method}(query?: {`);
        documentation.push(`${buildIdentation(identation + 1)}${params.map((param) => {
          const info = methods[method].queryParameters[param];

          return `${info.name}: ${getType(info)}`;
        }).join(`,\n${buildIdentation(identation + 1)}`)}`);
        documentation.push(`${buildIdentation(identation)}}, options?: { proxy: string, beforeSend: () => void, headers: object, body: object, baseUriParameters: object }): any;`);
      } else {
        documentation.push(`${buildIdentation(identation)}${method}(${getMethodInfo(method)}): any;`);
      }
    });

    return documentation;
  }

  return documentation;
}

function getMethodInfo(method) {
  const usesQuery = ['get', 'head'];

  if (usesQuery.indexOf(method) > -1) {
    return 'query?: any, options?: any, async?: boolean';
  }

  return 'body?: any, options?: any, async?: boolean';
}

function getType(info) {
  if (info.type && info.type[0]) {
    if (info.type[0] === 'integer') {
      return 'number';
    }

    if (['number', 'boolean', 'string', 'any'].some(type => type === info.type[0])) {
      return info.type[0];
    }
  }

  return 'any';
}

function buildIdentation(identation) {
  return new Array(identation + 1).join('  ');
}

==> raml-client-2.1.0.tgz: package/src/generator/documentation.js <==
module.exports = function buildDocumentation(name, ast) {
  const documentation = [];

  documentation.push('/* RAML Client API calls */');
  documentation.push(`declare var ${name}: {`);
  documentation.push('  /**');
  documentation.push('   * Make an API request to a custom URL.');
  documentation.push('   *');
  documentation.push('   * @param {string} url - Provide a url relative to the base URI.');
  documentation.push('   * @param {any} data - Provide a data object to replace template tags in the url.');
  documentation.push('   */');
  documentation.push('  (url: string, data?): {');
  documentation.push('    delete(body?: any, options?: any, async?: boolean): APIResponse,');
  documentation.push('    head(query?: any, options?: any, async?: boolean): APIResponse,');
  documentation.push('    get(query?: any, options?: any, async?: boolean): APIResponse,');
  documentation.push('    patch(body?: any, options?: any, async?: boolean): APIResponse,');
  documentation.push('    post(body?: any, options?: any, async?: boolean): APIResponse,');
  documentation.push('    put(body?: any, options?: any, async?: boolean): APIResponse');
  documentation.push('  };');

  buildMethods(ast.methods, documentation, 1);
  buildResources(ast.resources, documentation, 1);

  documentation.push('};');

  return documentation.join('\n');
};

function isValidVariableName(name) {
  return /^[a-zA-Z_$][0-9a-zA-Z_$]*$/.test(name);
}

function getVariableName(name) {
  if (isValidVariableName(name)) {
    return name;
  }

  return `"${name}"`;
}

function buildResources(resources, documentation, identation) {
  if (resources) {
    Object.keys(resources).forEach((resource) => {
      if (resource.startsWith('{')) {
        let methodName = resource.substring(1, resource.length - 1);

        if (methodName[0] === '+') {
          methodName = methodName.substring(1);
        }

        documentation.push(`${buildIdentation(identation)}${getVariableName(methodName)}(${getVariableName(methodName)}: string): {`);

        buildMethods(resources[resource].methods, documentation, identation + 1);
        buildResources(resources[resource].resources, documentation, identation + 1);

        documentation.push(`${buildIdentation(identation)}};`);
      } else {
        documentation.push(`${buildIdentation(identation)}${getVariableName(resource)}: {`);

        buildMethods(resources[resource].methods, documentation, identation + 1);
        buildResources(resources[resource].resources, documentation, identation + 1);

        documentation.push(`${buildIdentation(identation)}};`);
      }
    });

    return documentation;
  }

  return documentation;
}

function buildMethods(methods, documentation, identation) {
  if (methods) {
    Object.keys(methods).forEach((method) => {
      if (methods[method].queryParameters) {
        const params = Object.keys(methods[method].queryParameters);
        documentation.push(`${buildIdentation(identation)}/**`);
        documentation.push(`${buildIdentation(identation)} * ${methods[method].description}`);
        documentation.push(`${buildIdentation(identation)} *`);
        params.forEach((param) => {
          const info = methods[method].queryParameters[param];

          documentation.push(`${buildIdentation(identation)} * @param {${getType(info)}} ${info.name}${info.required ? '?' : ''} - ${info.description}`);
        });
        documentation.push(`${buildIdentation(identation)}*/`);
        documentation.push(`${buildIdentation(identation)}${getVariableName(method)}(query?: {`);
        documentation.push(`${buildIdentation(identation + 1)}${params.map((param) => {
          const info = methods[method].queryParameters[param];

          return `${info.name}: ${getType(info)}`;
        }).join(`,\n${buildIdentation(identation + 1)}`)}`);
        documentation.push(`${buildIdentation(identation)}}, options?: { proxy: string, beforeSend: () => void, headers: object, body: object, baseUriParameters: object }): APIResponse;`);
      } else {
        documentation.push(`${buildIdentation(identation)}${getVariableName(method)}(${getMethodInfo(method)}): APIResponse;`);
      }
    });

    return documentation;
  }

  return documentation;
}

function getMethodInfo(method) {
  const usesQuery = ['get', 'head'];

  if (usesQuery.indexOf(method) > -1) {
    return 'query?: any, options?: any, async?: boolean';
  }

  return 'body?: any, options?: any, async?: boolean';
}

function getType(info) {
  if (info.type && info.type[0]) {
    if (info.type[0] === 'integer') {
      return 'number';
    }

    if (['number', 'boolean', 'string', 'any'].some(type => type === info.type[0])) {
      return info.type[0];
    }
  }

  return 'any';
}

function buildIdentation(identation) {
  return new Array(identation + 1).join('  ');
}
